@@ -22,16 +22,34 @@
 #include <arpa/inet.h>
 #endif
 
+#include <stdint.h>
+
+
 #define ZIFACE_NAME_SIZE 16
 struct ziface_struct {
     char name[16];
     struct in_addr ip;
     struct in_addr netmask;
+#if defined(Z_MSC) || defined(SIOCGIFHWADDR)
+    unsigned char mac[6];
+#endif
 };
 
-int zifaces_get(struct ziface_struct *ifaces, int max_interfaces);
+int zifaces_get(struct ziface_struct *ifaces, int max_interfaces, int up_only);
 
 // 1 if input address is from local network
 int ziface_is_local(struct in_addr ia);
 
+
+
+struct zwifi_stat{
+    uint8_t raw_level, raw_noise, raw_qual, raw_updated;
+    uint8_t raw_max_qual;
+
+    int quality_percent, level_dbm;
+};
+
+int ziface_wifi_stats(struct zwifi_stat *stat, char *devname, int clear);
+char *ziface_macid(const char *namestarts);
+
 #endif
