@@ -1,6 +1,6 @@
 /*
     zsdl.h - SDL utilities
-    Copyright (C) 2011-2012 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2011-2020 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,22 +16,47 @@
 #ifdef Z_HAVE_SDL
 #include <SDL.h>
 
+#ifdef Z_HAVE_SDL2
+void SDL_UpdateRect(SDL_Surface *surface, int x, int y, int w, int h);
+#define KMOD_META KMOD_LALT
+#else
+#define SDL_Window SDL_Surface
+#define SDL_GetWindowWMInfo(w, i) SDL_GetWMInfo(i)
+#define SDL_Keycode SDLKey
+#define SDL_Keysym SDL_keysym
+#define SDL_Keymod SDLMod
+#define SDL_SetWindowIcon(w, i) SDL_WM_SetIcon(i, NULL)
+#define SDL_GetCurrentVideoDriver(s) SDL_VideoDriverName(s, sizeof(s)-1)
+
+#endif
+
 #include <ziconv.h>
 
 struct zzsdl {
+#ifdef Z_HAVE_SDL2
+	SDL_Window *window;
+	SDL_Renderer *renderer;
+#endif
+	SDL_Surface *screen;
+	Uint32 format;
+	int depth;
+
     void (*putpixel)(SDL_Surface *, int x, int y, int color);
     int font_w, font_h;
-    SDL_PixelFormat *format;
+    SDL_PixelFormat *pxformat;
 	int antialiasing, antialiasing_supported;
 	int inverse;
 	iconv_t to8b_cd; 
 	GString *to8b_gs;
 };
 
-extern struct zzsdl *zsdl;
+//extern struct zzsdl *zsdl;
 
 struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse);
 void zsdl_free(void);
+struct zzsdl *zsdl_instance();
+
+SDL_Surface *z_create_surface(struct zzsdl *zzsdl, int w, int h);
 
 void z_putpixel8(SDL_Surface *surface, int x, int y, int color);
 void z_putpixel16(SDL_Surface *surface, int x, int y, int color);
