@@ -19,7 +19,11 @@
 
 #include <math.h>
 
+static struct zzsdl *zsdl;
+
 struct zchart *zchart_init(SDL_Surface *surface, SDL_Rect *area, int bgcolor){
+	zsdl = zsdl_instance();
+
     struct zchart *chart = g_new0(struct zchart, 1);
 	chart->surface = surface;
 	memcpy(&chart->area, area, sizeof(SDL_Rect));
