@@ -153,9 +153,9 @@ static gpointer zfiledlg_thread_func(gpointer data){
 #ifdef Z_MSC_MINGW_CYGWIN
 
     if (!GetOpenFileName(&dlg->ofn)){
-		zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%x;X;%d", dlg, CommDlgExtendedError());
+		zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%lx;X;%d", (unsigned long)GPOINTER_TO_SIZE(dlg), CommDlgExtendedError());
 	}else{
-		zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%x;F;%s", dlg, dlg->ofn.lpstrFile);
+		zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%lx;F;%s", (unsigned long)GPOINTER_TO_SIZE(dlg), dlg->ofn.lpstrFile);
     }
 	g_free((void *)dlg->ofn.lpstrFilter);
 	return NULL;
@@ -192,10 +192,10 @@ static gpointer zfiledlg_thread_func(gpointer data){
 
     ret = gtk_dialog_run(GTK_DIALOG(dialog));
     if (ret != GTK_RESPONSE_ACCEPT){
-		zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%x;X;%d", dlg, ret);
+		zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%lx;X;%d", (unsigned long)GPOINTER_TO_SIZE(dlg), ret);
     }else{
         char *file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-        zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%x;F;%s", dlg, file);
+        zselect_msg_send(dlg->zsel, "ZFILEDLG;0x%lx;F;%s", (unsigned long)GPOINTER_TO_SIZE(dlg), file);
     }
     gtk_widget_destroy(dialog);
     while (gtk_events_pending()) gtk_main_iteration();
@@ -207,7 +207,9 @@ static gpointer zfiledlg_thread_func(gpointer data){
 }
 
 void zfiledlg_read_handler(int n, char *items[]){
+    dbg("zfiledlg_read_handler '%s'\n", items[1]);
     struct zfiledlg *dlg = (struct zfiledlg *)strtol(items[1], NULL, 16);
+    dbg("zfiledlg_read_handler %p\n", dlg);
 
 	if (strcmp(items[2], "F") == 0){
         g_free(dlg->last_dir);
