@@ -31,6 +31,10 @@
 
 #include <glib.h>
 
+#ifdef Z_HAVE_GNUTLS
+static gnutls_certificate_credentials_t xcred;
+#endif
+
 #ifdef Z_MSC
 #pragma warning(disable : 4996)
 #endif
@@ -42,6 +46,9 @@ static void zhttp_connected_handler(void *arg);
 static void zhttp_write_handler(void *arg);
 static void zhttp_read_handler(void *arg);
 static void zhttp_connecting_timer(void *arg);
+#ifdef Z_HAVE_GNUTLS
+static void zhttp_tls_handshake(void *arg);
+#endif
 
 struct zhttp *zhttp_init(void){
     struct zhttp *http = g_new0(struct zhttp, 1);
@@ -50,6 +57,8 @@ struct zhttp *zhttp_init(void){
 	http->state = ZHTTPST_NEW;
 	http->dataofs = -1;
 	http->cookies = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	http->headers = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free); /* TODO: who should free it? */
+	http->sock = -1;
 	return http;
 }
 
@@ -69,10 +78,14 @@ void zhttp_free(struct zhttp *http){
     g_free(http->page);
 	g_free(http->url);
 	g_hash_table_remove_all(http->cookies);
+	g_hash_table_remove_all(http->headers);
 
 	zhttp_post_free(http);
 	g_free(http->datastr);
 	if (http->connecting_timer_id > 0) zselect_timer_kill(http->zsel, http->connecting_timer_id);
+#ifdef Z_HAVE_GNUTLS
+	gnutls_deinit(http->session);
+#endif
 
 	g_free(http);
 }
@@ -94,20 +107,21 @@ void zhttp_post_free(struct zhttp *http){
 }
 
 void zhttp_disconnect(struct zhttp *http, char *errorstr){
-	if (http->sock < 0) return;
+	if (http->sock >= 0){
+		zselect_set(http->zsel, http->sock, NULL, NULL, NULL, NULL);
+		closesocket(http->sock);
+		http->sock = -1;
+	}
 	g_free(http->errorstr);
 	http->errorstr = errorstr;
 	if (http->errorstr) http->state = ZHTTPST_ERROR;
-	zselect_set(http->zsel, http->sock, NULL, NULL, NULL, NULL);
-	closesocket(http->sock);
-	http->sock = -1;
 	http_get_data(http);
 	if (http->callback) http->callback(http);
 }
 
 
 static void zhttp_prepare1(struct zhttp *http, struct zselect *zsel, const char *url, void *arg){
-    char *u, *c, *host;
+    char *u, *c, *host = NULL;
 
 	http->zsel = zsel;
 	http->arg = arg;
@@ -117,18 +131,39 @@ static void zhttp_prepare1(struct zhttp *http, struct zselect *zsel, const char
 	
     u = g_strdup(url);
 
-	c = z_strcasestr(u, "https:"); // we do not support https, try http
+	c = z_strcasestr(u, "https:"); 
 	if (c != NULL){
-		memmove(c + 4, c + 5, strlen(c + 4) + 1);
+#ifdef Z_HAVE_GNUTLS
+		zhttp_init_tls();
+		http->istls = 1;
+
+		c = z_strcasestr(u, "https://");
+		if (c){
+			host = c + strlen("https://");
+		}else{
+			host = u;
+		}
+		http->port = 443;
+#else
+		zhttp_disconnect(http, g_strdup(g_strdup("TLS not supported")));
+#endif
+	}else{
+		c = z_strcasestr(u, "http:");
+		if (c != NULL){
+			host = c + strlen("http://");
+		}else{
+			host = u;
+			c = z_strcasestr(u, "://");
+			if (c != NULL) zhttp_disconnect(http, g_strdup("Unsupported URL schema"));
+		}
+		http->port = 80;
+	}
+
+	if (http->state == ZHTTPST_ERROR) {
+		g_free(u);
+		return;
 	}
 
-    c = z_strcasestr(u, "http://");
-    if (c){
-        host = c + strlen("http://");
-    }else{
-        host = u;
-    }
-     
     c = strchr(host, '/');
     if (c){
         http->page = g_strdup(c);
@@ -141,14 +176,44 @@ static void zhttp_prepare1(struct zhttp *http, struct zselect *zsel, const char
     if (c){
         http->port = atoi(c + 1);
         *c = '\0';
-    }else{
-        http->port = 80;
     }
     http->server = g_strdup(host);
 
 	zbinbuf_truncate(http->request, 0);
 	zbinbuf_truncate(http->response, 0);
     g_free(u);
+
+#ifdef Z_HAVE_GNUTLS
+	if (http->istls){
+		int ret = gnutls_init(&http->session, GNUTLS_CLIENT);
+		if (ret != 0) {
+			zhttp_disconnect(http, g_strdup("Can't init TLS session"));
+			return;
+		}
+
+		ret = gnutls_server_name_set(http->session, GNUTLS_NAME_DNS, http->server, strlen(http->server));
+		if (ret != 0){
+			zhttp_disconnect(http, g_strdup("Can't set TLS server name"));
+			return;
+		}
+
+		ret = gnutls_set_default_priority(http->session);
+		if (ret != 0){
+			zhttp_disconnect(http, g_strdup("Can't set TLS default cipher priority"));
+			return;
+		}
+
+		ret = gnutls_credentials_set(http->session, GNUTLS_CRD_CERTIFICATE, xcred);
+		if (ret != 0){
+			zhttp_disconnect(http, g_strdup("Can't set TLS credentials"));
+			return;
+		}
+
+		gnutls_session_set_verify_cert(http->session, http->server, (gnutls_certificate_verify_flags)0);
+
+	}
+#endif
+
 }
 
 static void http_fill_cookies(gpointer key, gpointer value, gpointer user_data){
@@ -158,6 +223,15 @@ static void http_fill_cookies(gpointer key, gpointer value, gpointer user_data){
 	g_string_append_printf(gs, "%s=%s", (char*)key, (char*)value);
 }
 
+static void http_fill_user_headers(gpointer key, gpointer value, gpointer user_data){
+	zbinbuf_sprintfa((struct zbinbuf *)user_data, "%s: %s\r\n", key, value);
+	//GString *gs = g_string_new("");
+	//struct zbinbuf *zb = (struct zbinbuf *)user_data;
+	//g_string_append_printf(gs, "%s: %s\n", (char*)key, (char*)value);
+	//zbinbuf_append(zb, gs->str);
+	//g_string_free(gs, TRUE);
+}
+
 
 static void zhttp_headers(struct zhttp *http){
 
@@ -165,6 +239,9 @@ static void zhttp_headers(struct zhttp *http){
 	zbinbuf_append(http->request, "User-Agent: Libzia " Z_VERSION "\r\n");
 	zbinbuf_append(http->request, "Accept-Encoding:\r\n");
 	zbinbuf_append(http->request, "Connection: close\r\n");
+	if (g_hash_table_size(http->headers) > 0){
+		g_hash_table_foreach(http->headers, http_fill_user_headers, http->request);
+	}
 	if (g_hash_table_size(http->cookies) > 0){
 		GString *cookies = g_string_new("");
 		g_hash_table_foreach(http->cookies, http_fill_cookies, cookies);
@@ -190,12 +267,28 @@ static void zhttp_do(struct zhttp *http, struct zselect *zsel, void (*callback)(
 
 void zhttp_get(struct zhttp *http, struct zselect *zsel, const char *url, void (*callback)(struct zhttp *), void *arg){
 	zhttp_prepare1(http, zsel, url, arg);
+	if (http->state == ZHTTPST_ERROR){
+		callback(http);
+		return;
+	}
 	zbinbuf_sprintfa(http->request, "GET %s HTTP/1.0\r\n", http->page);
 	zhttp_headers(http);
 	zbinbuf_append(http->request, "\r\n");
 	zhttp_do(http, zsel, callback);
 }
 
+void zhttp_add_header(struct zhttp *http, const char *name, const char *value)
+{
+	g_hash_table_insert(http->headers, (char *)name, (char *)value);
+}
+
+void zhttp_raw(struct zhttp *http, struct zselect *zsel, const char *url, const char *raw_request,  void (*callback)(struct zhttp *), void *arg){
+	struct zbinbuf *zbb;
+
+	zhttp_prepare1(http, zsel, url, arg);
+	zbinbuf_append(http->request, raw_request);
+	zhttp_do(http, zsel, callback);
+}
 
 void zhttp_post(struct zhttp *http, struct zselect *zsel, const char *url, void (*callback)(struct zhttp *), void *arg){
 	struct zbinbuf *zbb;
@@ -216,6 +309,7 @@ void zhttp_post(struct zhttp *http, struct zselect *zsel, const char *url, void
 				char *contenttype = "application/octet-stream";
 				const char *ext = z_extension(var->filename);
 				if (strcasecmp(ext, ".png") == 0) contenttype = "image/png";
+				if (strcasecmp(ext, ".csv") == 0) contenttype = "text/plain";
 
 				zbinbuf_sprintfa(zbb, "Content-Disposition: form-data; name=\"%s\"; filename=\"%s\"\r\n", var->name, var->filename);
 				zbinbuf_sprintfa(zbb, "Content-Type: %s\r\n", contenttype);
@@ -288,6 +382,7 @@ static void zhttp_adns_callback(struct zasyncdns *adns, int n, int *family, int
     char errbuf[256];
 	GString *gs2;            
         
+	int port = http->port;
 
     if (errorstr != NULL){
         g_free(http->errorstr);
@@ -302,6 +397,8 @@ static void zhttp_adns_callback(struct zasyncdns *adns, int n, int *family, int
         http->callback(http);
         return;
     }
+
+
     
     for (i = 0; i < n; i++){
 //        dbg("socket(%d, %d, %d)\n", family[i], socktype[i], protocol[i]);
@@ -315,14 +412,16 @@ static void zhttp_adns_callback(struct zasyncdns *adns, int n, int *family, int
             http->sock = -1;
             continue;
         }
+
+
         
         switch (family[i]){
             case AF_INET:                
-                ((struct sockaddr_in *)(addr + i))->sin_port = htons(http->port);                
+                ((struct sockaddr_in *)(addr + i))->sin_port = htons(port);                
                 break;
 #ifdef AF_INET6            
             case AF_INET6:              
-                ((struct sockaddr_in6 *)(addr + i))->sin6_port = htons(http->port);                
+                ((struct sockaddr_in6 *)(addr + i))->sin6_port = htons(port);                
                 break;
 #endif            
             default:                   
@@ -337,6 +436,14 @@ static void zhttp_adns_callback(struct zasyncdns *adns, int n, int *family, int
 		http->serveraddr = g_strdup(gs2->str);
         g_string_free(gs2, TRUE);       
 
+
+	//addr[i].in.sin_addr.S_un.S_un_b.s_b1 = 127; //nefunguje
+	//addr[i].in.sin_addr.S_un.S_un_b.s_b2 = 0;
+	//addr[i].in.sin_addr.S_un.S_un_b.s_b3 = 0;
+	//addr[i].in.sin_addr.S_un.S_un_b.s_b4 = 1;
+	//((struct sockaddr_in *)(addr + i))->sin_port = htons(8888);
+
+
         ret = connect(http->sock, (struct sockaddr *)(addr + i), addrlen[i]);     
         if (ret < 0){           
             int err = z_sock_errno;         
@@ -375,9 +482,57 @@ static void zhttp_connected_handler(void *arg){
         return;
     }
 
+#ifdef Z_HAVE_GNUTLS
+	if (http->istls){
+		gnutls_transport_set_int(http->session, http->sock);
+		gnutls_handshake_set_timeout(http->session, GNUTLS_DEFAULT_HANDSHAKE_TIMEOUT);
+
+		zselect_set(http->zsel, http->sock, zhttp_tls_handshake, zhttp_tls_handshake, NULL, http);
+		http->state = ZHTTPST_TLS_HANDSHAKE;
+	}
+	else
+#endif
+	{
+		zselect_set(http->zsel, http->sock, NULL, zhttp_write_handler, NULL, http);
+		http->state = ZHTTPST_REQUEST;
+	}
+}
+
+#ifdef Z_HAVE_GNUTLS
+static void zhttp_tls_handshake(void *arg){
+	struct zhttp *http = (struct zhttp*)arg;
+
+	int ret = gnutls_handshake(http->session);
+	if (ret < 0 && gnutls_error_is_fatal(ret) == 0){
+		// like EWOULDBLOCK
+		return;
+	}
+	else if (ret < 0) {
+		if (ret == GNUTLS_E_CERTIFICATE_VERIFICATION_ERROR) {
+			/* check certificate verification status */
+			int type = gnutls_certificate_type_get(http->session);
+			unsigned status = gnutls_session_get_verify_cert_status(http->session);
+			gnutls_datum_t out;
+			int ret2 = gnutls_certificate_verification_status_print(status, type, &out, 0);
+			if (ret2 == 0){
+				zhttp_disconnect(http, g_strdup_printf("Cert verify failed: %s", out.data));
+			}else{
+				zhttp_disconnect(http, g_strdup_printf("Cert verify failed"));
+			}
+			gnutls_free(out.data);
+			return;
+		}
+		zhttp_disconnect(http, g_strdup_printf("Handshake failed: %s", gnutls_strerror(ret)));
+		return;
+	}
+	
+	char *desc = gnutls_session_get_desc(http->session);
+	dbg("Session info: %s\n", desc);
+	gnutls_free(desc);
 	zselect_set(http->zsel, http->sock, NULL, zhttp_write_handler, NULL, http);
 	http->state = ZHTTPST_REQUEST;
 }
+#endif
 
 static void zhttp_write_handler(void *arg){
     struct zhttp *http = (struct zhttp *)arg;
@@ -390,8 +545,15 @@ static void zhttp_write_handler(void *arg){
 	usleep(100000);
 #endif
 
+#ifdef Z_HAVE_GNUTLS
+	if (http->istls){
+		ret = gnutls_record_send(http->session, http->request->buf, tosend);
+	}else
+#endif
+	{
+		ret = send(http->sock, http->request->buf, tosend, 0);
+	}
 
-	ret = send(http->sock, http->request->buf, tosend, 0);
     //dbg("send(%d) = %d\n", tosend, ret);
 	if (ret <= 0){
 		zhttp_disconnect(http, g_strdup_printf("Error writing http socket: %s", z_sock_strerror()));
@@ -407,6 +569,15 @@ static void zhttp_write_handler(void *arg){
 		zselect_set(http->zsel, http->sock, zhttp_read_handler, NULL, NULL, http);
 		http->state = ZHTTPST_HEADER;
 	}
+
+#ifdef Z_HAVE_GNUTLS
+	if (http->istls){
+		size_t pending = gnutls_record_check_pending(http->session);
+		if (pending > 0){
+			zhttp_read_handler(arg);
+		}
+	}
+#endif
 }
 
 #define ZHTTP_BUFSIZE 4096
@@ -425,7 +596,18 @@ static void zhttp_read_handler(void *arg){
 	usleep(100000);
 #endif
 
-	ret = recv(http->sock, buf, torecv, 0);
+#ifdef Z_HAVE_GNUTLS
+	if (http->istls){
+		ret = gnutls_record_recv(http->session, buf, torecv);
+		if (ret < 0 && gnutls_error_is_fatal(ret) == 0) {
+			return;
+		}
+	}else
+#endif
+	{
+		ret = recv(http->sock, buf, torecv, 0);
+	}
+
 	if (ret < 0){
 		zhttp_disconnect(http, g_strdup_printf("Error reading http socket: %s", z_sock_strerror()));
 		return;
@@ -622,3 +804,30 @@ void zhttp_post_json(struct zhttp *http, struct zselect *zsel, const char *url,
 
 	zhttp_do(http, zsel, callback);
 }
+
+#ifdef Z_HAVE_GNUTLS
+
+static int zgnutls_initialized = 0;
+
+void zhttp_init_tls(void){
+
+	if (zgnutls_initialized) return;
+	zgnutls_initialized = 1;
+
+	if (sizeof(ssize_t) != sizeof(size_t)) zinternal("sizeof(ssize_t) != sizeof(size_t)  %d != %d", sizeof(ssize_t), sizeof(size_t));
+
+	if (gnutls_check_version("3.5.8") == NULL) zinternal("GnuTLS 3.5.8 or later is required");
+	int ret = gnutls_global_init();
+	if (ret) zinternal("Can't init gnutls");
+
+	ret = gnutls_certificate_allocate_credentials(&xcred);
+	if (ret) zinternal("Can't allocate certificate credentials");
+
+	ret = gnutls_certificate_set_x509_system_trust(xcred);
+	if (ret < 0) zinternal("Can't set the system trusted CAs");
+
+
+	
+}
+
+#endif
\ No newline at end of file
