@@ -1,6 +1,6 @@
 /*
     zloc.c - Maidenhead WW locator functions
-    Copyright (C) 2002-2012  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2020  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -20,6 +20,9 @@
 
 #undef LOCDEBUG
 
+#define sqr(x) ((x)*(x))       
+
+
 static void tostr(GString *gs, double x){
     int a,b;
 
@@ -37,7 +40,7 @@ static void tostr(GString *gs, double x){
         x=b-x;
     x*=60;
 
-    g_string_sprintfa(gs, "%d.%d'%3.1f\"", a, b, x);
+    g_string_append_printf(gs, "%d.%d'%3.1f\"", a, b, x);
 }
 
 
@@ -73,11 +76,11 @@ double qth(char *qth,int width) {
         d=tolower(*c);
         if (d<'a' || d>'x') return(-100);
         res+=(d-97)*M_PI/2160.0;
+	    res+=M_PI/4320.0;
     }else{
         if (width&2) res+=M_PI/180;
     }
     
-    res+=M_PI/4320.0;
     
     /*if (strlen(c)>=8){
         c+=2;
@@ -116,15 +119,15 @@ int qrbqtf(char *myqth,char *recqth,double *qrb,double *qtf,GString *gs, int fla
 
     if (gs!=NULL){
 		g_string_truncate(gs, 0);
-        g_string_sprintfa(gs, "From: ");
+        g_string_append_printf(gs, "From: ");
 		tostr(gs, h1);
-		g_string_sprintfa(gs, "E  ");
+		g_string_append_printf(gs, "E  ");
 		tostr(gs, w1);
-		g_string_sprintfa(gs, "N  to ");
+		g_string_append_printf(gs, "N  to ");
 		tostr(gs, h2);
-		g_string_sprintfa(gs, "E  ");
+		g_string_append_printf(gs, "E  ");
 		tostr(gs, w2);
-		g_string_sprintfa(gs, "N");
+		g_string_append_printf(gs, "N");
     }
 
     ret = hw2qrbqtf(h1, w1, h2, w2, qrb, qtf);
@@ -157,6 +160,54 @@ int hw2qrbqtf(double h1, double w1, double h2, double w2, double *qrb, double *q
     return 0;
 }
 
+void hw2km(double h1, double w1, double h2, double w2, int *kx, int *ky){
+	double qrb, qtf;
+    
+	hw2qrbqtf(h1, w1, h2, w2, &qrb, &qtf);
+/*    dbg("qrb=%f qtf=%f\n", qrb, qtf);*/
+    *kx = (int)(  qrb*sin(qtf));
+    *ky = (int)(- qrb*cos(qtf));
+}
+
+void hw2km_d(double h1, double w1, double h2, double w2, double *kx, double *ky){
+	double qrb, qtf;
+    
+	hw2qrbqtf(h1, w1, h2, w2, &qrb, &qtf);
+/*    dbg("qrb=%f qtf=%f\n", qrb, qtf);*/
+
+    *kx =   qrb*sin(qtf);
+    *ky = - qrb*cos(qtf);
+}
+
+void hw2km_f(double h1, double w1, double h2, double w2, float *fkx, float *fky){
+	double kx, ky;
+	hw2km_d(h1, w1, h2, w2, &kx, &ky);
+
+	/*if (ikx < -32768) ikx = -32768;
+	if (ikx > 32767) ikx = 32767;
+	if (iky < -32768) iky = -32768;
+	if (iky > 32767) iky = 32767;*/
+
+	*fkx = (float)kx;
+	*fky = (float)ky;
+}
+
+void km2qrbqtf(int kx, int ky, double *qrb, double *qtf){
+    int dx, dy;
+    
+    dx=kx;
+    dy=ky;
+    
+	if (qrb){
+		*qrb=sqrt((double)(sqr(dx)+sqr(dy)));
+	}
+	if (qtf){
+		*qtf=atan2((double)dx,(double)-dy);   
+		if (*qtf<0) *qtf+=2*M_PI;
+	}
+/*    dbg("km2qrbqtf(%d, %d)=%d, %d\n",kx, ky, (int)*qrb, (int)(*qtf*180.0/M_PI));*/
+}
+
 /*************** REVERSE QRB ***************************/
 #define EPSILON 0.00000000005 
 /* qtf in radian */
@@ -416,3 +467,4 @@ int main(int argc,char *argv[]){
 } 
 
 #endif
+
