@@ -1,6 +1,6 @@
 /*
     zsdl.c - reusable SDL functions
-    Copyright (C) 2011-2018 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2011-2020 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -59,7 +59,7 @@
 #include <windows.h>
 #endif
 
-struct zzsdl *zsdl = NULL;
+static struct zzsdl *zsdl = NULL;
 
 #ifdef Z_MSC
 #pragma warning(disable : 4996)
@@ -73,8 +73,18 @@ struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse){
 
     if (zsdl) zsdl_free();
     zsdl = g_new0(struct zzsdl, 1);
+	zsdl->screen = screen;
 	zsdl->inverse = inverse;
 
+#ifdef Z_HAVE_SDL2
+	zsdl->renderer = SDL_CreateSoftwareRenderer(zsdl->screen);
+	if (!zsdl->renderer) zinternal("Can't alloc software renderer");
+
+	SDL_SetRenderDrawColor(zsdl->renderer, 0xFF, 0xFF, 0xFF, 0xFF);
+	SDL_RenderClear(zsdl->renderer);
+
+#endif
+
     switch(screen->format->BytesPerPixel){
         case 1:
             zsdl->putpixel = inverse ? z_putpixel8inv : z_putpixel8;
@@ -97,7 +107,7 @@ struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse){
 
     zsdl->font_h = font_h;
 	zsdl->font_w = zsdl_h2w(font_h);
-    zsdl->format = screen->format;
+    zsdl->pxformat = screen->format;
 
 	outline9x16 = zfont_create_outline(font9x16, sizeof(font9x16), 16);
 
@@ -116,7 +126,10 @@ struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse){
 void zsdl_free(void){
     if (!zsdl) return;
 	g_free(outline9x16);
-
+#ifdef Z_HAVE_SDL2
+	if (zsdl->renderer != NULL) SDL_DestroyRenderer(zsdl->renderer);
+	if (zsdl->pxformat != NULL) SDL_FreeFormat(zsdl->pxformat);
+#endif
 	if (zsdl->to8b_cd != NULL) iconv_close(zsdl->to8b_cd);
 	zsdl->to8b_cd = NULL;
 	g_string_free(zsdl->to8b_gs, TRUE);
@@ -125,6 +138,10 @@ void zsdl_free(void){
     zsdl = NULL;
 }
 
+struct zzsdl *zsdl_instance(){
+	return zsdl;
+}
+
 int zsdl_h2w(int font_h){
 	switch (font_h){
 		case 13:
@@ -146,6 +163,34 @@ int zsdl_h2w(int font_h){
 	}
 }
 
+SDL_Surface *z_create_surface(struct zzsdl *zsdl, int w, int h){
+	SDL_Surface *surface;
+
+	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h,
+		zsdl->pxformat->BitsPerPixel,
+		zsdl->pxformat->Rmask,
+		zsdl->pxformat->Gmask,
+		zsdl->pxformat->Bmask,
+	    zsdl->pxformat->Amask);
+	return surface;
+}
+
+#ifdef Z_HAVE_SDL2
+void SDL_UpdateRect(SDL_Surface *surface, int x, int y, int w, int h){
+	/*static int iii = 0;
+	SDL_Rect r;
+	r.x = iii * 10;
+	r.y = iii * 10;
+	r.w = 10;
+	r.h = 10;
+	SDL_FillRect(zsdl->screen, &r, rand());
+	iii++; */
+
+	//SDL_RenderPresent(zsdl->renderer);
+	SDL_UpdateWindowSurface(zsdl->window);
+}
+#endif
+
 int zsdl_max_font_h(int w, int h, char *text){
 	int ww;
 	int l = strlen(text);
@@ -1102,7 +1147,7 @@ void z_pip(SDL_Surface *surface, int x, int y, int color1, int color2, int zoom)
 int z_makecol(int r, int g, int b){
     SDL_PixelFormat *format;
 
-    format = zsdl->format;
+    format = zsdl->pxformat;
     return (((r>>format->Rloss)<<format->Rshift)&format->Rmask) |
            (((g>>format->Gloss)<<format->Gshift)&format->Gmask) |
            (((b>>format->Bloss)<<format->Bshift)&format->Bmask);
@@ -1140,10 +1185,10 @@ void z_dumpbitmap(SDL_Surface *screen, SDL_Surface *surface){
     r.h = surface->h;
     
     backup = SDL_CreateRGBSurface(SDL_SWSURFACE, surface->w, surface->h, 
-                        zsdl->format->BitsPerPixel,                               
-                        zsdl->format->Rmask, 
-                        zsdl->format->Gmask, 
-                        zsdl->format->Bmask, 
+                        zsdl->pxformat->BitsPerPixel,                               
+                        zsdl->pxformat->Rmask, 
+                        zsdl->pxformat->Gmask, 
+                        zsdl->pxformat->Bmask, 
                         0);
     
     SDL_BlitSurface(screen, &r, backup, NULL);
@@ -1376,8 +1421,8 @@ int zsdl_sys_key_repeat(int delay, int rate){
 		}
 	}
 #endif
-#ifdef Z_HAVE_SDL
-	SDL_EnableKeyRepeat(delay, rate);
+#ifdef Z_HAVE_SDL1
+	SDL_EnableKeyRepeat(delay, rate);	// not in 2.0
 #endif
 	return 0;
 }
@@ -1455,7 +1500,7 @@ int zsdl_maximize(struct zzsdl *zsdl, int mode){ // 0=normal, 1=maximize, 2=togg
 	WINDOWPLACEMENT plc;
 
 	SDL_VERSION(&info.version);
-	SDL_GetWMInfo(&info);
+	SDL_GetWindowWMInfo(zsdl->window, &info);	
 	switch (mode){
 		case 0: 
 			cmd = SW_RESTORE;
@@ -1463,11 +1508,22 @@ int zsdl_maximize(struct zzsdl *zsdl, int mode){ // 0=normal, 1=maximize, 2=togg
 		case 2:
 			memset(&plc, 0, sizeof(plc));
 			plc.length = sizeof(plc);
+#ifdef Z_HAVE_SDL1
 			if (!GetWindowPlacement(info.window, &plc)) break;
+#endif
+#ifdef Z_HAVE_SDL2
+			if (!GetWindowPlacement(info.info.win.window, &plc)) break;
+#endif
+
 			if (plc.showCmd == SW_MAXIMIZE) cmd = SW_RESTORE;
 			break;
 	}
+#ifdef Z_HAVE_SDL1
 	ShowWindow(info.window, cmd);
+#endif
+#ifdef Z_HAVE_SDL2
+	ShowWindow(info.info.win.window, cmd);
+#endif
 #endif
 	return 0;
 }
@@ -1479,10 +1535,16 @@ int zsdl_maximized(struct zzsdl *zsdl, int *w, int *h){
 	WINDOWPLACEMENT plc;
 
 	SDL_VERSION(&info.version);
-	SDL_GetWMInfo(&info);
+	SDL_GetWindowWMInfo(zsdl->window, &info);
 	memset(&plc, 0, sizeof(plc));
 	plc.length = sizeof(plc);
+#ifdef Z_HAVE_SDL1
 	if (!GetWindowPlacement(info.window, &plc)) return 0;
+#endif
+#ifdef Z_HAVE_SDL2
+	if (!GetWindowPlacement(info.info.win.window, &plc)) return 0;
+#endif
+
 	if (w != NULL) *w = plc.rcNormalPosition.right - plc.rcNormalPosition.left;
 	if (h != NULL) *h = plc.rcNormalPosition.bottom - plc.rcNormalPosition.top;
 	return (plc.showCmd == SW_MAXIMIZE);
