@@ -13,12 +13,15 @@
 #include <eprintf.h>
 #include <zerror.h>
 #include <zselect.h>
+#include <zfence.h>
 #include <zsock.h>
 #include <zdebug.h>
 #include <zlist.h>
+#include <zthread.h>
 #include <ztime.h>
 
 #include <errno.h>
+#define _USE_MATH_DEFINES // for MSVC
 #include <math.h>
 #include <stdarg.h>
 #include <string.h>
@@ -89,13 +92,11 @@ struct zselect{
     struct ztimer timers;
     struct bottom_half bottom_halves;
 
+	MUTEX_DEFINE(fdsets);
     fd_set w_read;
     fd_set w_write;
     fd_set w_error;
-
-    fd_set x_read;
-    fd_set x_write;
-    fd_set x_error;
+	// end of mutex protected data
     
     int w_max;
     int timer_id;
@@ -109,6 +110,7 @@ struct zselect{
     int profile;
     double limit;
 	char hint[100];
+    int select_running; // run from thread
 };
 
 #ifdef Z_UNIX_ANDROID
@@ -134,6 +136,7 @@ struct zselect *zselect_init(void (*redraw)(void)){
     init_list(zsel->timers);
     init_list(zsel->bottom_halves);
     zsel->timer_id = 1;
+	MUTEX_INIT(zsel->fdsets);
 
     FD_ZERO(&zsel->w_read);
     FD_ZERO(&zsel->w_write);
@@ -161,7 +164,8 @@ struct zselect *zselect_init(void (*redraw)(void)){
 void zselect_free(struct zselect *zsel){
     
     zselect_set(zsel, zsel->msgpipe[0], NULL, NULL, NULL, zsel);
-    z_pipe_close(zsel->msgpipe[0]);
+	MUTEX_FREE(zsel->fdsets);
+	z_pipe_close(zsel->msgpipe[0]);
     z_pipe_close(zsel->msgpipe[1]);
     g_string_free(zsel->msg_recv, TRUE);
 }
@@ -263,6 +267,7 @@ static void zselect_timers_check(struct zselect *zsel)
 			if (zsel->profile) gettimeofday(&start, NULL);
 			//dbg("%s\n", tt->fname);
             tt->func(tt->arg);
+			//zfence_check(tt->fname);
 			if (zsel->profile) zselect_handle_profile(zsel, &start, tt->func, tt->fname);
             g_free(tt);
             CHK_BH;
@@ -436,21 +441,19 @@ void zselect_set_dbg(struct zselect *zsel, int fd, void (*read_func)(void *), ch
     zfd->error_func = error_func;
 	zfd->error_fname = error_fname;
     zfd->arg = arg;
-    if (read_func) FD_SET(fd, &zsel->w_read);
-    else {
-        FD_CLR(fd, &zsel->w_read);
-        FD_CLR(fd, &zsel->x_read);
-    }
+
+
+	MUTEX_LOCK(zsel->fdsets);
+	if (read_func) FD_SET(fd, &zsel->w_read);
+    else FD_CLR(fd, &zsel->w_read);
+   
     if (write_func) FD_SET(fd, &zsel->w_write);
-    else {
-        FD_CLR(fd, &zsel->w_write);
-        FD_CLR(fd, &zsel->x_write);
-    }
+    else FD_CLR(fd, &zsel->w_write);
+    
     if (error_func) FD_SET(fd, &zsel->w_error);
-    else {
-        FD_CLR(fd, &zsel->w_error);
-        FD_CLR(fd, &zsel->x_error);
-    }
+    else FD_CLR(fd, &zsel->w_error);
+	MUTEX_UNLOCK(zsel->fdsets);
+
 
 #ifdef Z_MSC_MINGW
 	if (!read_func && !write_func && !error_func){
@@ -484,11 +487,11 @@ void zselect_set_read(struct zselect *zsel, int fd, void (*read_func)(void *), v
     zfd->read_func = read_func;
 	zfd->arg = arg;
    
+	MUTEX_LOCK(zsel->fdsets);
 	if (read_func) FD_SET(fd, &zsel->w_read);
-    else {
-        FD_CLR(fd, &zsel->w_read);
-        FD_CLR(fd, &zsel->x_read);
-    }
+    else FD_CLR(fd, &zsel->w_read);
+	MUTEX_UNLOCK(zsel->fdsets);
+
 
 #ifdef Z_MSC_MINGW
 	if (!read_func && !zfd->write_func && !zfd->error_func){
@@ -514,11 +517,12 @@ void zselect_set_write(struct zselect *zsel, int fd, void (*write_func)(void *),
     zfd->write_func = write_func;
 	zfd->arg = arg;
    
+	MUTEX_LOCK(zsel->fdsets);
 	if (write_func) FD_SET(fd, &zsel->w_write);
-    else {
-        FD_CLR(fd, &zsel->w_write);
-        FD_CLR(fd, &zsel->x_write);
-    }
+    else FD_CLR(fd, &zsel->w_write);
+    if (zsel->select_running) zselect_msg_send_raw(zsel, "\n"); 
+	MUTEX_UNLOCK(zsel->fdsets);
+
 
 #ifdef Z_MSC_MINGW
 	if (!zfd->read_func && !write_func && !zfd->error_func){
@@ -647,6 +651,11 @@ static int check_signals(struct zselect *zsel)
 void zselect_loop(struct zselect *zsel)
 {
 	int err;
+	fd_set x_read;
+	fd_set x_write;
+	fd_set x_error;
+
+
 
     CHK_BH;
     while (!zsel->terminate) {
@@ -674,10 +683,13 @@ void zselect_loop(struct zselect *zsel)
             tm = &tv;
         }
 
-        memcpy(&zsel->x_read, &zsel->w_read, sizeof(fd_set));
-        memcpy(&zsel->x_write, &zsel->w_write, sizeof(fd_set));
-        memcpy(&zsel->x_error, &zsel->w_error, sizeof(fd_set));
-        if (zsel->terminate) break;
+		MUTEX_LOCK(zsel->fdsets);
+        memcpy(&x_read, &zsel->w_read, sizeof(fd_set));
+        memcpy(&x_write, &zsel->w_write, sizeof(fd_set));
+        memcpy(&x_error, &zsel->w_error, sizeof(fd_set));
+		MUTEX_UNLOCK(zsel->fdsets);
+
+		if (zsel->terminate) break;
         if (!zsel->w_max && list_empty(zsel->timers)) break;
 #ifdef Z_UNIX
         critical_section = 1;
@@ -714,7 +726,15 @@ void zselect_loop(struct zselect *zsel)
 #endif	
      /*   ST_STOP;
         sound(0);  */
-        n = select(zsel->w_max, &zsel->x_read, &zsel->x_write, &zsel->x_error, tm);
+	    MUTEX_LOCK(zsel->fdsets);
+        zsel->select_running = 1;
+	    MUTEX_UNLOCK(zsel->fdsets);
+
+        n = select(zsel->w_max, &x_read, &x_write, &x_error, tm);
+
+	    MUTEX_LOCK(zsel->fdsets);
+        zsel->select_running = 0;
+	    MUTEX_UNLOCK(zsel->fdsets);
        /* sound(800);
         ST_START;      */
         if (n < 0) {
@@ -731,11 +751,11 @@ void zselect_loop(struct zselect *zsel)
 				
                 FD_ZERO(&used);
                 g_string_append(gs, "\nR:");
-                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_read)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
+                for (i = 0; i < 256; i++) if (FD_ISSET(i, &x_read)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
                 g_string_append(gs, "\nW:");
-                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_write)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
+                for (i = 0; i < 256; i++) if (FD_ISSET(i, &x_write)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
                 g_string_append(gs, "\nE:");
-                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_error)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
+                for (i = 0; i < 256; i++) if (FD_ISSET(i, &x_error)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
 
                 for (i = 0; i < 256; i++) {
                     if (!FD_ISSET(i, &used)) continue;
@@ -778,31 +798,34 @@ void zselect_loop(struct zselect *zsel)
         while (n > 0 && ++i < zsel->w_max) {
             int k = 0;
 			struct zselect_fd *zfd = get_fd_index(zsel, i);
-			if (FD_ISSET(zfd->sock, &zsel->x_read)) {
+			if (FD_ISSET(zfd->sock, &x_read)) {
                 if (zfd->read_func) {
 					if (zsel->profile) gettimeofday(&start, NULL);
 					//dbg("%s\n", zfd->read_fname);
                     zfd->read_func(zfd->arg);
+					//zfence_check(zfd->read_fname);
 					if (zsel->profile) zselect_handle_profile(zsel, &start, zfd->read_func, zfd->read_fname);
                     CHK_BH;
                 }
                 k = 1;
             }
-            if (FD_ISSET(zfd->sock, &zsel->x_write)) {
+            if (FD_ISSET(zfd->sock, &x_write)) {
                 if (zfd->write_func) {
 					if (zsel->profile) gettimeofday(&start, NULL);
 					//dbg("%s\n", zfd->write_fname);
-                    zfd->write_func(zfd->arg);
+					zfd->write_func(zfd->arg);
+					//zfence_check(zfd->write_fname);
 					if (zsel->profile) zselect_handle_profile(zsel, &start, zfd->write_func, zfd->write_fname);
                     CHK_BH;
                 }
                 k = 1;
             }
-            if (FD_ISSET(zfd->sock, &zsel->x_error)) {
+            if (FD_ISSET(zfd->sock, &x_error)) {
                 if (zfd->error_func) {
 					if (zsel->profile) gettimeofday(&start, NULL);
 					//dbg("%s\n", zfd->error_fname);
-                    zfd->error_func(zfd->arg);
+					zfd->error_func(zfd->arg);
+					//zfence_check(zfd->error_fname);
 					if (zsel->profile) zselect_handle_profile(zsel, &start, zfd->error_func, zfd->error_fname);
                     CHK_BH;
                 }
