@@ -1,7 +1,7 @@
 /*
     zserial_ftdi - portable serial port API
 	Module for FTDI USB converters using libftdi
-    Copyright (C) 2012 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2012-2020 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -34,21 +34,21 @@ static int zserial_ftdi_open(struct zserial *zser){
 
     zser->ftdi = ftdi_new();
     if (!zser->ftdi){
-        g_string_sprintf(zser->errorstr, "Can't alloc ftdi context (out of memory?)");
+        g_string_printf(zser->errorstr, "Can't alloc ftdi context (out of memory?)");
         zserial_close(zser);
         return -1;
     }
 
     if (zser->serial && *zser->serial){
         if (ftdi_usb_open_desc(zser->ftdi, zser->vid, zser->pid, NULL, zser->serial)){
-            g_string_sprintf(zser->errorstr, "Can't open FTDI device %04X:%04X-%s : %s", zser->vid, zser->pid, zser->serial, ftdi_get_error_string(zser->ftdi));  
+            g_string_printf(zser->errorstr, "Can't open FTDI device %04X:%04X-%s : %s", zser->vid, zser->pid, zser->serial, ftdi_get_error_string(zser->ftdi));  
             zserial_close(zser);
             return -1;
         }
         zser->id = g_strdup_printf("%04X:%04X-%s", zser->vid, zser->pid, zser->serial);
     }else{
         if (ftdi_usb_open(zser->ftdi, zser->vid, zser->pid)){
-            g_string_sprintf(zser->errorstr, "Can't open FTDI device %04X:%04X : %s", zser->vid, zser->pid, ftdi_get_error_string(zser->ftdi));  
+            g_string_printf(zser->errorstr, "Can't open FTDI device %04X:%04X : %s", zser->vid, zser->pid, ftdi_get_error_string(zser->ftdi));  
             zserial_close(zser);
             return -1;
         }
@@ -86,7 +86,7 @@ static int zserial_ftdi_read(struct zserial *zser, void *data, int len, int time
     while(!ztimeout_occured(tmo)){
         r = ftdi_read_data(zser->ftdi, (unsigned char*)data, len);
         if (r < 0) { // usb device unavailable or usb_bulk_read failed
-            g_string_sprintf(zser->errorstr, "Can't read from %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));
+            g_string_printf(zser->errorstr, "Can't read from %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));
             zserial_close(zser);
             return -1;
         }
@@ -103,7 +103,7 @@ static int zserial_ftdi_write(struct zserial *zser, void *data, int len){
 
     ret = ftdi_write_data(zser->ftdi, (unsigned char *)data, len);
     if (ret < 0){
-        g_string_sprintf(zser->errorstr, "Can't write to %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));
+        g_string_printf(zser->errorstr, "Can't write to %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));
         zserial_close(zser);
         return -1;
     }
@@ -118,7 +118,7 @@ static int zserial_ftdi_close(struct zserial *zser){
 
 static int zserial_ftdi_dtr(struct zserial *zser, int on){
     if (ftdi_setdtr(zser->ftdi, on)){ // TODO polarity
-        g_string_sprintf(zser->errorstr, "Can't set DTR on %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));  
+        g_string_printf(zser->errorstr, "Can't set DTR on %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));  
         return -1;
     }
     return 0;
@@ -127,7 +127,7 @@ static int zserial_ftdi_dtr(struct zserial *zser, int on){
 static int zserial_ftdi_rts(struct zserial *zser, int on){
     dbg("zserial_rts(zser->ftdi=%p\n", zser->ftdi);
     if (ftdi_setrts(zser->ftdi, on)){ // TODO polarity
-        g_string_sprintf(zser->errorstr, "Can't set RTS on %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));  
+        g_string_printf(zser->errorstr, "Can't set RTS on %s: %s", zser->id, ftdi_get_error_string(zser->ftdi));  
         return -1;
     }
     return 0;
